﻿using Adam.Core.DataMapper;
using Adam.Core.Records;
using Adam.Core.Search;
using Adam.Web;
using Adam.Web.Extensions.UI.Controls;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using Adam.Web.Tools.Fluent;
using Adam.Web.UI;
using Adam.Web.UI.Controls;
using Image = System.Web.UI.WebControls.Image;
using Adam.Web.Extensions.Providers.Actions;

namespace Adam.Fadel.Web.Actions
{
    public class AssetRightsClearanceAction : ModalDialogAction
    {
        private LinkButton _ClearanceButton;
        private Label _ClearanceResultLabel;
        private string[] _Filenames;
        private DropDownList _FormatDropDownList;
        private DatePickerBox _InDatePickerBox;
        private DatePickerBox _OutDatePickerBox;
        private string[] _RecordIds;
        private RecordCollection _Records;
        private Table _ResultTable;
        private DropDownList _TerritoryDropDownList;

        protected override string DefaultLabel
        {
            get { return "Asset Rights Clearance..."; }
        }

        private void EnsureRecordIds()
        {
            _Records = new RecordCollection(AdamContext.GetApplication());

            var options = new RecordLoadOptions();
            options.CheckSecurity = false;
            options.LockMode = LockMode.ReadUncommitted;
            options.PreLoadFiles = PreLoadFile.FilesVersions;

            object dataItem = ResolveDataItem();

            var record = dataItem as Record;
            if (record != null)
            {
                _Records.Add(record);
            }

            var basket = dataItem as Basket;
            if (basket != null)
            {
                _Records.Load(basket.Items, options);
            }

            var selectionGroup = dataItem as SelectionGroup;
            if (selectionGroup != null)
            {
                _Records.Load(selectionGroup, options);
            }

            //Remove records without files
            var recordsToRemove = new List<Guid>();
            foreach (Record item in _Records)
            {
                if (item.Files.Count == 0) recordsToRemove.Add(item.Id);
            }
            foreach (Guid item in recordsToRemove)
            {
                _Records.Remove(item);
            }

            //Extract record ids and associated filenames
            var recordIds = new List<Guid>();
            var filenames = new List<string>();
            foreach (Record item in _Records)
            {
                recordIds.Add(item.Id);
                filenames.Add(item.Files.LatestMaster.FileName);
            }
            _RecordIds = recordIds.Select(id => id.ToString()).ToArray();
            _Filenames = filenames.ToArray();
        }

        protected override DialogUpdatePanel CreateDialog()
        {
            var dialog = new DynamicDialogPanel();
            dialog.CreateDialogContent += OnCreateDialogContent;
            return dialog;
        }

        private void OnCreateDialogContent(object sender, CreateDialogContentEventArgs e)
        {
            EnsureRecordIds();

            Control parent = e.Container;

            parent.AddGeneric("div", div =>
            {
                div.AddClass("dialog-header");
                div.InnerText = "ASSET RIGHTS CLEARANCE";
            });

            parent.AddControl(new ModalActionWithViewStateContainer(), container =>
            {
                // Generate controls that contain state.
                container.AddGeneric("div", div =>
                {
                    div.AddClass("dialog-body dialog-resizable");

                    CreateBodyContent(div);
                });

                container.AddGeneric("div", div =>
                {
                    div.AddClass("dialog-footer");

                    div.AddControl(new Image
                    {
                        ImageUrl = "~/Style/5300/Images/Fadel_Logo_s.png",
                        CssClass = "fadel_logo"
                    });

                    _ClearanceButton = div.AddControl(new LinkButton
                    {
                        ID = "_ClearanceButton",
                        CssClass = "button default-button ok-button",
                        Text = "Get Clearance"
                    });
                    _ClearanceButton.Click += _ClearanceButton_Click;

                    div.AddControl(new LinkButton
                    {
                        CssClass = "button cancel-button",
                        Text = "Close",
                        CausesValidation = false,
                        OnClientClick = "Adam.Controls.Dialog.findDialog(this).hide();return false;"
                    });
                });
            });
        }

        private void CreateBodyContent(HtmlGenericControl control)
        {
            control.AddGeneric("div", div =>
            {
                //div.AddClass("dialog-body dialog-resizable");

                Table table = div.AddControl(new Table());
                table.AddClass("arc-table");
                var row = new TableRow();
                var cell = new TableCell();

                //Asset ID
                cell = new TableCell();
                cell.Controls.Add(new Label
                {
                    Text = (_RecordIds.Length > 1) ? "Asset IDs" : "Asset ID"
                });
                row.Cells.Add(cell);
                cell = new TableCell();
                cell.ColumnSpan = 2;
                cell.Controls.Add(new Label
                {
                    Text = string.Join(", ", _RecordIds),
                    Width = Unit.Pixel(350)
                });
                row.Cells.Add(cell);
                table.Rows.Add(row);

                //Filename
                row = new TableRow();
                cell = new TableCell();
                cell.Controls.Add(new Label
                {
                    Text = (_Filenames.Length > 1) ? "Filenames" : "Filename"
                });
                row.Cells.Add(cell);
                cell = new TableCell();
                cell.ColumnSpan = 2;
                cell.Controls.Add(new Label
                {
                    Text = string.Join(", ", _Filenames),
                    Width = Unit.Pixel(350)
                });
                row.Cells.Add(cell);
                table.Rows.Add(row);

                //Format
                row = new TableRow();
                cell = new TableCell();
                cell.Controls.Add(new Label
                {
                    Text = "Format"
                });
                row.Cells.Add(cell);
                cell = new TableCell();
                cell.ColumnSpan = 2;
                _FormatDropDownList = new DropDownList
                {
                    ID = "FormatDropDownList",
                    ClientIDMode = ClientIDMode.Static,
                    EnableViewState = true,
                    Width = Unit.Pixel(350)
                };
                AddItems(_FormatDropDownList, "Format");
                cell.Controls.Add(_FormatDropDownList);
                row.Cells.Add(cell);
                table.Rows.Add(row);

                //Territory
                row = new TableRow();
                cell = new TableCell();
                cell.Controls.Add(new Label
                {
                    Text = "Territory"
                });
                row.Cells.Add(cell);
                cell = new TableCell();
                cell.ColumnSpan = 2;
                _TerritoryDropDownList = new DropDownList
                {
                    ID = "TerritoryDropDownList",
                    ClientIDMode = ClientIDMode.Static,
                    EnableViewState = true,
                    Width = Unit.Pixel(350)
                };
                AddItems(_TerritoryDropDownList, "Territory");
                cell.Controls.Add(_TerritoryDropDownList);
                row.Cells.Add(cell);
                table.Rows.Add(row);

                //In Date
                row = new TableRow();
                cell = new TableCell();
                cell.Controls.Add(new Label
                {
                    Text = "Date"
                });
                row.Cells.Add(cell);
                cell = new TableCell();
                cell.Width = Unit.Pixel(150);
                _InDatePickerBox = new DatePickerBox
                {
                    ID = "InDatePickerBox",
                    ClientIDMode = ClientIDMode.Static,
                    EnableViewState = true,
                    WatermarkText = "From",
                    Width = Unit.Pixel(150)
                };
                cell.Controls.Add(_InDatePickerBox);

                row.Cells.Add(cell);
                cell = new TableCell();
                _OutDatePickerBox = new DatePickerBox
                {
                    ID = "OutDatePickerBox",
                    ClientIDMode = ClientIDMode.Static,
                    EnableViewState = true,
                    WatermarkText = "To",
                    Width = Unit.Pixel(150)
                };
                cell.Controls.Add(_OutDatePickerBox);
                row.Cells.Add(cell);
                table.Rows.Add(row);

                //Clearance Result
                row = new TableRow();
                cell = new TableCell();
                _ClearanceResultLabel = new Label
                {
                    Text = "Clearance&nbsp;Result",
                    Visible = true
                };
                cell.Controls.Add(_ClearanceResultLabel);
                row.Cells.Add(cell);
                cell = new TableCell();
                cell.ColumnSpan = 2;
                _ResultTable = new Table();
                _ResultTable.CssClass = "fadel_resulttable";
                cell.Controls.Add(_ResultTable);
                row.Cells.Add(cell);
                table.Rows.Add(row);
            });
        }


        private void _ClearanceButton_Click(object sender, EventArgs e)
        {
            Dictionary<string, ClearanceResultInfo> result = GetClearanceResult(_RecordIds);

            for (int i = 0; i < _Filenames.Length; i++)
            {
                var row = new TableRow();
                var cell = new TableCell();
                var image = new ImageBox();
                image.SizeMode = ImageBoxSizeMode.Zoom;
                image.Width = Unit.Pixel(30);
                image.Height = Unit.Pixel(30);
                image.ImageAlign = ImageAlign.Middle;
                image.ImageLocation = ImageBoxLocation.MiddleCenter;
                image.ImageUrl = RecordWebResource.GetImageUrl(_Records[new Guid(_RecordIds[i])]);
                cell.Controls.Add(image);
                row.Cells.Add(cell);
                cell = new TableCell();
                cell.Text = _Filenames[i];
                cell.Style[HtmlTextWriterStyle.TextOverflow] = "ellipsis";
                cell.Style[HtmlTextWriterStyle.Overflow] = "hidden";
                cell.Style[HtmlTextWriterStyle.WhiteSpace] = "nowrap";
                cell.Style["max-width"] = "150px";
                row.Cells.Add(cell);
                cell = new TableCell();
                cell.CssClass = "button menu-item";

                cell.ForeColor = Color.White;

                cell.Width = Unit.Pixel(220);
                if (!result.ContainsKey(_RecordIds[i]))
                {
                    cell.BackColor = Color.Red;
                    cell.Text = "RECORD NOT FOUND";
                    cell.ToolTip = "The record appears to have been removed from the system";
                }
                else
                {
                    ClearanceResultInfo info = result[_RecordIds[i]];
                    switch (info.Status)
                    {
                        case ClearanceResultStatus.Available:
                            cell.BackColor = Color.Green;
                            cell.Text = "AVAILABLE";
                            break;
                        case ClearanceResultStatus.NotAvailable:
                            cell.BackColor = Color.Red;
                            cell.Text = "NOT AVAILABLE";
                            break;
                        case ClearanceResultStatus.AvailableWithExceptions:
                            cell.BackColor = Color.Gold;
                            cell.Text = "AVAILABLE WITH EXCEPTIONS";
                            cell.ToolTip = info.AdditionalInfo;
                            break;
                        default:
                            break;
                    }
                }
                row.Cells.Add(cell);
                _ResultTable.Rows.Add(row);
            }
            ScriptManager.RegisterStartupScript(_ClearanceButton, typeof (AssetRightsClearanceAction), "resizeBody",
                "resizeDialogToContent('" + _ClearanceButton.ClientID + "')", true);
        }

        private Dictionary<string, ClearanceResultInfo> GetClearanceResult(string[] assetIds)
        {
            var helper = new FadelHelper(AdamContext.GetApplication());
            DateTime? InDate = null;
            DateTime? inDateToDate = null;
            DateTime? OutDate = null;
            DateTime? outDateToDate = null;

            if (_InDatePickerBox.SelectedDate.HasValue)
                InDate =
                    _InDatePickerBox.SelectedDate.Value.ToDateTime();
            if (_OutDatePickerBox.SelectedDate.HasValue)
                OutDate =
                    _OutDatePickerBox.SelectedDate.Value.ToDateTime();

            return helper.GetClearanceResult(assetIds,
                _FormatDropDownList.SelectedValue,
                _TerritoryDropDownList.SelectedValue,
                InDate,
                inDateToDate,
                OutDate,
                outDateToDate);
        }

        private void AddItems(DropDownList dropDownList, string attributeName)
        {
            var helper = new FadelHelper(AdamContext.GetApplication());
            var result = new Dictionary<int, string>();

            switch (attributeName.ToUpperInvariant())
            {
                case "FORMAT":
                    result = helper.GetFormats();
                    break;
                case "TERRITORY":
                    result = helper.GetTerritories();
                    break;
                default:
                    break;
            }

            foreach (int key in result.Keys)
            {
                dropDownList.Items.Add(new ListItem(result[key], key.ToString()));
            }
        }

        protected override ActionSupportResult IsSupported(object dataItem, ActionSupportCache supportCache)
        {
            var record = dataItem as Record;
            if ((record != null) && (record.Files.Count > 0))
            {
                return ActionSupportResult.Enabled;
            }

            var basket = dataItem as Basket;
            if (basket != null)
            {
                return basket.Items.Count > 0 ? ActionSupportResult.Enabled : ActionSupportResult.Disabled;
            }

            var group = dataItem as SelectionGroup;
            if (group != null)
            {
                return group.Count > 0 ? ActionSupportResult.Enabled : ActionSupportResult.Disabled;
            }

            return ActionSupportResult.Disabled;
        }
    }
}