﻿using Adam.Core;
using Adam.Core.Records;
using Adam.Web;
using Adam.Web.Extensions.Providers.Actions;
using Adam.Web.Extensions.UI.Controls;
using Adam.Web.Tools.Fluent;
using Adam.Workflow.Core.Client;
using Adam.Workflow.Shared;
using System;
using System.Collections.Generic;
using System.Web.UI;
using System.Web.UI.WebControls;
using Adam.Core.Search;
using System.Linq;

namespace CustomActions.Customization
{
    class CustomWorkflowAction : ModalDialogAction
    {
        private TextBox _commentsTextBox;
        private DropDownList _workflowDDL;

        protected override DialogUpdatePanel CreateDialog()
        {
            var dialog = new DynamicDialogPanel();
            dialog.CreateDialogContent += OnCreateDialogContent;
            return dialog;
        }

        private void OnCreateDialogContent(object sender, CreateDialogContentEventArgs e)
        {
            Control container = e.Container;
            var panel = container.Parent as DynamicDialogPanel;
            if (panel != null)
            {
                panel.Attributes["width"] = "700px";
                panel.Attributes["height"] = "700px";
            }

            container.AddGeneric("div", div =>
            {
                div.AddClass("dialog-header");
                div.InnerText = "Start Worfklow";
            });

            Application app = AdamContext.GetApplication();
            WorkflowDefinitionCollection definitions = new WorkflowDefinitionCollection(app);
            definitions.Load(new SearchExpression("Status = ?", new object[] { WorkflowDefinitionConstants.StatusType.Released }));

            container.AddGeneric("div", div =>
            {
                div.AddClass("dialog-body dialog-resizable");

                div.AddControl(new Label
                {
                    Text = "Select Workflow..."
                });

                _workflowDDL = div.AddControl(new DropDownList
                {
                    ID = "WorkflowDDL",
                    ClientIDMode = ClientIDMode.Static,
                    Width = Unit.Pixel(300),
                    DataSource = definitions,
                    DataValueField = "Id",
                    DataTextField = "Name"
                });
            });

            container.AddGeneric("div", div =>
            {
                div.AddClass("dialog-body dialog-resizable");

                div.AddControl(new Label
                {
                    Text = "Please add some comments"
                });

                _commentsTextBox = div.AddControl(new TextBox
                {
                    ID = "CommentsTextBox",
                    ClientIDMode = ClientIDMode.Static,
                    Width = Unit.Pixel(300)
                });
            });

            container.AddGeneric("div", div =>
            {
                div.AddClass("dialog-footer");
                LinkButton button = div.AddControl(new LinkButton
                {
                    CssClass = "button default-button ok-button",
                    Text = "Start",
                    CausesValidation = true
                });
                button.Click += OnAcceptDialog;
                div.AddControl(new LinkButton
                {
                    CssClass = "button cancel-button",
                    Text = "Cancel",
                    CommandName = "Hide",
                    CausesValidation = false
                });
            });
        }

        private void OnAcceptDialog(object sender, EventArgs e)
        {
            try
            {
                var acceptButton = (LinkButton)sender;
                Dictionary<string, object> parameters = new Dictionary<string, object>();

                Application app = AdamContext.GetApplication();

                IList<Guid> ids = new List<Guid>();
                var record = ResolveDataItem() as Record;
                if (record != null)
                {
                    ids.Add(record.Id);
                }
                else
                {
                    var basket = ResolveDataItem() as Basket;
                    if (basket != null)
                    {
                        ids = basket.Items.Select(id => id).ToList();
                    }
                    else
                    {
                        var selectionGroup = ResolveDataItem() as SelectionGroup;
                        if (selectionGroup != null)
                        {
                            ids = selectionGroup.Select(id => id).ToList();
                        }
                    }
                }

                parameters.Add(WorkflowConstants.ReferencedItems, string.Join(";", ids.Select(item => item)));
                parameters.Add(WorkflowConstants.ReferencedItemType, "Record");

                WorkflowDefinition definition = new WorkflowDefinition(app);
                definition.LoadReleased("General Metadata Update");
                //definition.LoadReleased(_workflowDDL.SelectedItem.Value.ToString());
                parameters.Add("SelectedWorkflow", _workflowDDL.SelectedValue.ToString());
                parameters.Add("Comment", _commentsTextBox.Text);
                definition.StartProcess(parameters);

                // Raise the bubble event to hide the dialog.
                acceptButton.CommandName = DialogUpdatePanel.HideCommandName;
            }
            catch (Exception ex)
            {
                AdamContext.Log(ex);
                ShowError(ex);
            }
        }

        protected override string DefaultLabel
        {
            get
            {
                return "Start Workflow";
            }
        }

        protected override ActionSupportResult IsSupported(object dataItem, ActionSupportCache supportCache)
        {
            if (!AdamContext.GetApplication().HasRole(WorkflowDefinitionConstants.RoleType.Start.ToSetting()))
            {
                return ActionSupportResult.Hidden;
            }

            Record record = dataItem as Record;
            if (record != null)
            {
                return ActionSupportResult.Enabled;
            }

            Basket basket = dataItem as Basket;
            if (basket != null)
            {
                return basket.Items.Count > 0 ? ActionSupportResult.Enabled : ActionSupportResult.Disabled;
            }

            SelectionGroup group = dataItem as SelectionGroup;
            if (group != null)
            {
                return group.Count > 0 ? ActionSupportResult.Enabled : ActionSupportResult.Disabled;
            }

            return ActionSupportResult.Disabled;
        }
    }
}
